"""Responsible for all the Covid data handling functionality."""

import configparser
import csv
from typing import Tuple
from uk_covid19 import Cov19API
import json
import sched
import time
# from main import update_covid_data_global

CONFIG = configparser.ConfigParser()
CONFIG.read('config.ini')
LOCAL_LOCATION = CONFIG['SETTINGS']['local_location']
NATION_LOCATION = CONFIG['SETTINGS']['nation_location']

covid_data = {'local_7day_infections': '',
              'national_7day_infections': '',
              'hospital_cases': '',
              'deaths_total': ''}
# covid_data = {}
s = sched.scheduler(time.time, time.sleep)

def parse_csv_data(csv_filename: str) -> list:
    """Takes a csv file name as an input, then reads the
    file into a list.

    Args:
        csv_filename (str): A string containing the filename for
            the local csv file holding covid data.
            e.g. 'nation_2021-10-28.csv'

    Returns:
        data_list (list): Returns a list of lists, as read from the
            csv file.
    """
    with open(csv_filename, mode='r') as csvfile:
        data = csv.reader(csvfile)
        data_list = list(data)
    return data_list


def process_covid_csv_data(covid_csv_data: list) -> Tuple[int, int, int]:
    """Processed the csv_data, returning various data items.
 
    Args:
        covid_csv_data (list): The list of covid data returned from
            parse_csv_data.

    Returns:
        last7days_cases (int): National cumulative total of covid-19
            infections in the last 7 days.
        current_hospital_cases (int): The total number of people
            hospitalised because of covid-19 in the given country.
        total_deaths (int): The total number of people who died within
            30 days of a positive covid-19 test in the given country.
    """
    last7days_cases = 0
    # last7days_cases starts counting from 3 instead of zero, as
    # the first row is column titles, and the following data entries
    # are either missing or not-yet reliable.
    for i in range(3, 10):
        last7days_cases += int(covid_csv_data[i][6])
    current_hospital_cases = int(covid_csv_data[1][5])
    # total_deaths returns the most recent death toll in the csv file
    total_deaths = int(covid_csv_data[14][4])
    return last7days_cases, current_hospital_cases, total_deaths


def covid_API_request(
        location: str="Exeter",
        location_type: str="ltla")-> dict:
    """This function takes two arguments, location, and
    location_type.  It then returns various covid data for
    that location in the form of a dictionary.

    Args:
        location (str): The location about which covid-19 statistics
            are being requested (more details above).
            location_type can be either: overview, nation, region,
            nhsRegion, utla (upper-tier local authority), or
            ltla (lower-tier local authority).
            More details can be found at:
            https://coronavirus.data.gov.uk/details/developers-guide/
            main-api#params-filters
        location_type (str): Location must be a place name 
            corresponding to the location type, i.e. "England" for
            nation, or "Cambridge" for ltla.

    Returns:
        data (dict): A dictionary containing the output data from
            the API request.
    """
    # Constructs a list of strings for the filters
    # of the API request.  It uses this function's arguments
    # as the input data.
    user_filters = [
        "areaName=" + location,
        "areaType=" + location_type
        ]
    # Constructs a structure for the API request, specifying
    # the desired variables, and how to order them in the
    # dictionary which it will subsequently return.
    selected_covid_data = {
        "areaCode": "areaCode",
        "areaName": "areaName",
        "areaType": "areaType",
        "date": "date",
        "cumDailyNsoDeathsByDeathDate": "cumDailyNsoDeathsByDeathDate",
        "hospitalCases": "hospitalCases",
        "newCasesBySpecimenDate": "newCasesBySpecimenDate"
    }
    selected_api = Cov19API(
        filters=user_filters,
        structure=selected_covid_data,
    )
    # Gets the data file in the form of a .json file under
    # the parameters specified in user_filters and
    # the selected_covid_data structure.
    data = selected_api.get_json()
    return(data)


def process_new_local_data(updated_local_dict: dict) -> int:
    """Takes the up-to-date local covid dictionary data returned from
    covid_api_request and returns the 7 day infection rate.

    Args:
        update_local_dict (dict): Holds the .json dictionary
            returned from covid_API_request.

    Returns:
        last7days_cases (int): Local (ltla) cumulative total of covid-19
            infections in the last 7 days.
    """
    # Extracts the data portion of the .json dictionary.
    # This is in the form of a list of dictionaries.
    # Each dictionary represents the data for that day.
    updated_data_list = updated_local_dict['data']
    last7days_cases = 0
    # Sums the new cases for the last seven days.
    for i in range(7):
        data_item = updated_data_list[i]['newCasesBySpecimenDate']
        last7days_cases += data_item
    return last7days_cases


def process_new_national_data(updated_national_dict: dict) -> Tuple[
        int, int, int]:
    """Takes the up-to-date national covid dictionary data returned from
    covid_api_request and returns various covid data items.

    Note that processing is slightly different to local data, as the data
    from the covid API is formatted differently when returning
    national data.

    Args:
        updated_national_dict (dict): Holds the .json dictionary
            returned from covid_API_request.

    Returns:
        last7days_cases (int): National cumulative total of covid-19
            infections in the last 7 days.
        current_hospital_cases (int): The total number of people
            hospitalised because of covid-19 in the given country.
        total_deaths (int): The total number of people who died within
            30 days of a positive covid-19 test in the given country.
    """
    updated_data_list = updated_national_dict['data']
    last7days_cases = 0
    # Sums the new cases for the last seven days.
    # NB: the first two data items for cases as they are blank, and
    # incomplete respectively.
    for i in range(2,9):
        data_item = updated_data_list[i]['newCasesBySpecimenDate']
        last7days_cases += data_item
    current_hospital_cases = updated_data_list[1]['hospitalCases']
    # Death count is not included in the data for recent days,
    # so the program searches through daily data until a valid count
    # is found.
    while updated_data_list[i]['cumDailyNsoDeathsByDeathDate'] is None:
        i += 1
    total_deaths = updated_data_list[i]['cumDailyNsoDeathsByDeathDate']
    return last7days_cases, current_hospital_cases, total_deaths


def schedule_covid_updates(update_interval: int, update_name: str) -> None:
    """Updates the covid data at the given interval.

    Args:
        update_interval  (int): The number of seconds until the 
            function is scheduled to trigger.
        update_name (str): The update's name/title. This is for
            visual, and debugging  purposes only.
    """
    print(update_name + ": Covid update in " + str(update_interval))
    s.enter(update_interval, 1, update_covid_data)
    s.run(blocking=False)


def update_covid_data() -> None:
    """Updates the global dictionary containing the covid data

    Queries the Public Health England API for two dictionaries
    of data, containing local, and national data respectively.
    These are then parsed into the relevant processor functions,
    and stored in the global covid_data dictionary, ready to be
    outputted to the render_template.
    NB: hospital_cases and deaths_total have identifiers concatenated
    onto them as the index.html file does not feature any for those
    data items.
    """
    updated_local_dict = covid_API_request(
        LOCAL_LOCATION, "ltla")
    updated_national_dict = covid_API_request(
        NATION_LOCATION, "nation")
    global covid_data
    covid_data['local_7day_infections'] = \
        process_new_local_data(updated_local_dict)
    covid_data['national_7day_infections'], \
        covid_data['hospital_cases'], \
        covid_data['deaths_total'] = \
        process_new_national_data(updated_national_dict)
    covid_data['hospital_cases'] = 'National hospital cases: ' \
                                   + str(covid_data['hospital_cases'])
    covid_data['deaths_total'] = 'National death toll: ' \
                                 + str(covid_data['deaths_total'])
    print("Covid data updated")


def covid_data_getter() -> dict:
    return covid_data
