"""Returns all the news stories that mention COVID-19"""

import configparser
import requests
import sched

# news_list = []

def check_headlines(news: dict,
                    news_blacklist: list,
                    num_of_articles: int = 4) -> list:
    """Takes the headlines and returns a list of articles which are
    not on the blacklist.
    
    Args:
        news (dict): A dictionary returned from the news_API_request function.
        news_blacklist (list): A list of strings containing the headlines of
            articles which the user has elected to hide from the webpage.
        num_of_articles (int):  is the number of articles to return.
            The default number is set to 4 (which returns 5 articles
            as counting starts from 0. This number was chosen as it is the
            general number that fits on most displays easily.

    Returns:
        cleared_articles_list (list): Returns a list of dictionaries
            holding information pertaining to articles (as described in
            parse_article_details).
    """
    # Extracts the 'articles' list of dictionaries from the main
    # dictionary, and puts it into its own list.
    articles = news['articles']
    cleared_articles_list = []
    counter = 0
    # This loop runs until the desired number of non-blacklisted
    # articles have been found.
    while len(cleared_articles_list) <= num_of_articles:
        # Takes the individual article dictionary from the list
        # of articles. This conatins the relevant information for
        # the dashboard.
        article = parse_article_details(articles[counter])
        # Compares the title of the given article against all of the
        # titles of articles which have been hidden (blacklisted).
        if is_article_blacklisted(article['title'], news_blacklist) is False:
            # Adds the valid article to the list.
            cleared_articles_list.append(article)
        # Increments the counter to continue searching through the
        # list of articles.
        counter += 1
    return cleared_articles_list


def parse_article_details(article: dict) -> dict:
    """Takes the article dictionary and extracts the relevant
    information.

    Args:
        article (dict): The article will be passed into the function 
            as a dictionary adhereing the structure of the following
            example:
            {'source': {'id': 'bbc-news', 'name': 'BBC News'},
            'author': 'https://www.facebook.com/bbcnews',
            'title': 'Covid-19 origins may never be known,
            US intelligence agencies say',
            'description': 'But US agencies say the virus, first
            identified in China, was not developed as a 
            biological weapon.',
            'url': 'https://www.bbc.co.uk/news/
            world-us-canada-59100114',
            'urlToImage': 'https://ichef.bbci.co.uk/news/1024/
            branded_news/F895/production/
            _121273636_gettyimages-1195315493.jpg',
            'publishedAt': '2021-10-30T03:47:01Z',
            'content': 'Image source, Getty Images\r\nImage caption, The
            virus was first identified in the Chinese city of Wuhan in
            2019\r\nUS intelligence agencies say they may never be able
            to identify the origins of Covid-19… [+2315 chars]'}

        article_details (dict):  This article would then be returned as
            the following:
            {'title': 'BBC News: Covid-19 origins may never be known,
            US intelligence agencies say',
            'content': https://www.bbc.co.uk/news/
            world-us-canada-59100114'}

    """
    # Concatenates the title of the source with the article's title
    title = str(article['source']['name']) + ': ' + str(article['title'])
    content = article['url']
    article_details = {'title': title, 'content': content}
    return article_details


def is_article_blacklisted(article_title: str, news_blacklist) -> bool:
    """Compares the title of a given article against a list of strings
    containing the titles of articles that the user has elected to hide
    (blacklist).

    If the title of the given article is in the list of blacklisted
    titles, it returns True. If it is not blacklisted, it returns false.
    """
    for blacklisted_title in news_blacklist:
        if article_title == blacklisted_title:
            return True
    return False


def news_API_request(
        covid_terms: str="Covid COVID-19 coronavirus") -> dict:
    """Returns articles from News API that contain one or more
    of the search terms in the headline.

    These articles are returned as a dictionary.
    Search terms are a string, with each term separated by
    a space.
    """
    API_url = construct_API_request_url(covid_terms)
    response = requests.get(API_url).json()
    return response


def construct_API_request_url(queries: str) -> str:
    """Constructs a URL for the request to NewsAPI

    Takes the queries which article headlines must contain, as
    well as details from the local config file and constructs
    the URL for the query, returning it as a string.
    """
    config = configparser.ConfigParser()
    config.read('config.ini')
    base_url = config['URL DETAILS']['base_url']
    queries = terms_to_queries(queries)
    api_key = config['URL DETAILS']['news_api_key']
    language = config['SETTINGS']['language']
    complete_url = base_url \
        + "apiKey=" + api_key \
        + "&language=" + language \
        + "&qInTitle=" + queries \
        + "&sortBy=publishedAt"
    return complete_url


def terms_to_queries(search_terms: str) -> str:
    """Takes discrete search terms (separated by a space)
    and converts them into the format usable for the News API
    search query.
    """
    queries = search_terms.replace(" ", " OR ")
    return queries


def schedule_news_updates(update_interval: int, update_name: str) -> None:
    """Updates the news at the given interval.

    Args:
        update_interval (int): The number of seconds until
            the function is scheduled to trigger.
        update_name (str): The update's name/title. This is for
            visual, and debugging  purposes only.
    """
    print(update_name + ": News update in " + str(update_interval))
    s.enter(update_interval, 1, update_news)


def update_news(article_blacklist: list) -> list:
    """Updates the news list by querying for new articles, and filtering
    out blacklisted ones.
    """
    print("News updated.")
    api_request = news_API_request(
        "Covid COVID-19 coronavirus")
    news_list = check_headlines(api_request,
                                article_blacklist,
                                4)
    return news_list

def covid_news_getter() -> list:
    return news_list

# def update_news(news_blacklist: list,
#                 queries: str = "",
#                 num_of_articles: int = 4) -> list:
#     """Returns an updated list of news articles

#     Takes some search queries and returns articles with those terms
#     in the headline of the article, provided that the article is not
#     in the blacklist.
#     The number of articles returned is equal to the num_of_articles
#     variable.
#     """
#     print("News updated.")
#     news = news_API_request(queries)
#     cleared_articles = check_headlines(news,
#                                        news_blacklist,
#                                        num_of_articles)
#     # print(cleared_articles)
#     return cleared_articles
