"""This is the main file for the Covid Data Dashboard

This is responsible for taking information from the various
data handling files and collating it into the user interface.
It is also responsible for taking user inputs and passing them
to the relevant functions.
"""

import configparser
import sched
import time
from flask import Flask
from flask import request
from flask import render_template
import covid_news_handling
import covid_data_handler
from time_handler import calc_sched_time

CONFIG = configparser.ConfigParser()
CONFIG.read('config.ini')
LOCAL_LOCATION = CONFIG['SETTINGS']['local_location']
NATION_LOCATION = CONFIG['SETTINGS']['nation_location']
CSV_FILENAME = CONFIG['SETTINGS']['csv_filename']

updates = []
# news = []
article_blacklist = []
# covid_data = {'local_7day_infections': '',
#               'national_7day_infections': '',
#               'hospital_cases': '',
#               'deaths_total': ''}

s = sched.scheduler(time.time, time.sleep)
app = Flask(__name__)


@app.route('/index')
def index():
    """Displays the main page information for the dashboard

    Responsible for collecting the various data items which need
    displaying, processes them (if neccessary), and returns them
    into the Flask render template to be displayed on the webpage.

    Returns:
        render_template (function): returns the name and variables for
            the flask render_template to display the webpage.
    """
    news_article_to_hide = request.args.get('notif')
    article_blacklist.append(news_article_to_hide)
    news = covid_news_handling.update_news(article_blacklist)
    # update_covid_data()
    text_field = request.args.get('two')
    if text_field:
        update_time = request.args.get('update')
        schedule_update(update_time, text_field)
    s.run(blocking=False)
    covid_data_handler.update_covid_data()
    covid_data = covid_data_handler.covid_data_getter()
    print(covid_data)
    return render_template('index.html',
                           title='Coronavirus Data Dashboard',
                           location=LOCAL_LOCATION,
                           local_7day_infections=covid_data[
                               'local_7day_infections'],
                           nation_location=NATION_LOCATION,
                           national_7day_infections=covid_data[
                               'national_7day_infections'],
                           hospital_cases=covid_data['hospital_cases'],
                           deaths_total=covid_data['deaths_total'],
                           news_articles=news,
                           updates=updates,
                           image='favicon.png')


def schedule_update(update_time: str, update_name: str) -> None:
    """Primary function for the scheduling of events.

    Handles the update request by taking the request.args data,
    and then initiates the associated function depending on the type
    of update (news or data).
    If both news and data are requested, two separate updates
    are created.
    It also passes the information to be visually appended to the
    updates list.

    Args:
        update_time (str): The time at which the update is to be
            scheduled. Takes the format HH:MM. This time is converted
            into a delay in seconds when it is passed into the
            devolved update schedules.
        update_name (str): The update's name/title. This is for
            visual, and debugging purposes only.
    """
    update_time_sec = calc_sched_time(update_time)
    update_request_covid_data = request.args.get('covid-data')
    update_request_news = request.args.get('news')
    if update_request_covid_data:
        covid_data_handler.schedule_covid_updates(update_time_sec, update_name)
        append_updates_list(
            update_time, update_name, update_request_covid_data)
    if update_request_news:
        schedule_news_updates(update_time_sec, update_name)
        append_updates_list(update_time, update_name, update_request_news)


def append_updates_list(
        update_time: str,
        update_name: str,
        update_type: str) -> None:
    """Adds a scheduled update to the updates list to be
    visually displayed on the webpage.

    Details the type of update, it's name/title, as well as the time
    at which it is scheduled to occur.

    Args:
        update_time (str): The time at which the update is scheduled
            to occur (in the format HH:MM).
        update_name (str): The update's name/title. This is for
            visual, and debugging  purposes only.
        update_type (str): The type of update being requested.
            This will be either 'covid-data' or 'news'.
    """
    if update_type == 'covid-data':
        update_content = "Covid update at "
    elif update_type == 'news':
        update_content = "News update at "
    update_content += update_time
    update = {'title': update_name, 'content': update_content}
    updates.append(update)


def schedule_news_updates(update_interval: int, update_name: str) -> None:
    """Updates the news at the given interval.

    Args:
        update_interval (int): The number of seconds until
            the function is scheduled to trigger.
        update_name (str): The update's name/title. This is for
            visual, and debugging  purposes only.
    """
    print(update_name + ": News update in " + str(update_interval))
    s.enter(update_interval, 1, update_news_global)


def update_news_global() -> None:
    """This is used to update the news list

    It calls the update_news function from the covid_news_handling
    module, which returns a list. This is then assigned to the
    global variable in this module called 'news'. news is then handled
    by the index() function to be displayed on the webpage.
    """
    global news
    news = covid_news_handling.update_news()


# def update_covid_data() -> None:
#     """Updates the global dictionary containing the covid data

#     Queries the Public Health England API for two dictionaries
#     of data, containing local, and national data respectively.
#     These are then parsed into the relevant processor functions,
#     and stored in the global covid_data dictionary, ready to be
#     outputted to the render_template.
#     NB: hospital_cases and deaths_total have identifiers concatenated
#     onto them as the index.html file does not feature any for those
#     data items.
#     """
#     updated_local_dict = covid_data_handler.covid_API_request(
#         LOCAL_LOCATION, "ltla")
#     updated_national_dict = covid_data_handler.covid_API_request(
#         NATION_LOCATION, "nation")
#     covid_data['local_7day_infections'] = \
#         covid_data_handler.process_new_local_data(updated_local_dict)
#     covid_data['national_7day_infections'], \
#         covid_data['hospital_cases'], \
#         covid_data['deaths_total'] = \
#         covid_data_handler.process_new_national_data(updated_national_dict)
#     covid_data['hospital_cases'] = 'National hospital cases: ' \
#                                    + str(covid_data['hospital_cases'])
#     covid_data['deaths_total'] = 'National death toll: ' \
#                                  + str(covid_data['deaths_total'])
#     print("Covid data updated")


# def update_covid_data_global() -> None:
#     covid_data = covid_data_handler.covid_data_getter()


if __name__ == '__main__':
    app.run()
